/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.Breadcrumbs");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.string.BidiComplex");

(function(){

var BidiComplex = dojox.string.BidiComplex;

dojo.declare("jazz.ui.Breadcrumbs",	[dijit._Widget, dijit._Templated], {
	// summary:
	//		A breadcrumb widget which accepts an array of name/href pairs.
	//
	//      NOTE: This is provisional API. Clients using this API do so with the
	//		awareness that this class may be removed or substantially changed at
	//		future times without warning.
	// description:
	//		This is a simple widget which manages the dom structure for a list
	//		of breadcrumb segments. It provides API to set the list of breadcrumbs
	//		as well as to read and update it again later.
	// segments:
	//		An array of objects representing the segments in the breadcrumbs. It can
	//		be passed in the constructor to the widget. It can also be read and updated
	//		at any time during the lifecycle of the widget. The refresh function must be
	//		called after updating the segments to have the changes reflected in the dom.
	//		Each segment in the array must contain a 'label' attribute which will be
	//		added to the dom as text (not html). Each segment must also contain one of
	//		'href' or 'onClick'. If both 'href' and 'onClick' are provided, the 'href'
	//		will be placed on the anchor node, so it shows in the status bar of the browser
	//		but will be suppressed in favour of the provided onClick handler. The actual
	//		click event will be passed to the onClick handler.
	// since:
	//		0.7
	
	segments: [],
	
	refresh: function() {
		// summary:
		//		Refreshes the content of the widget.
		// description:
		//		Should be called after any change to the segments attribute. This will
		//		cause the widget to update its dom based on the current segments.
		while (this.domNode.firstChild)
			this.domNode.removeChild(this.domNode.firstChild);
		for (var i = 0; i < this.segments.length; i++) {
			this.domNode.appendChild(this._createCrumb(this.segments[i]));
		}
	},
	
	// INTERNAL PROPERTIES AND FUNCTIONS
	
	templateString: "<div class='jazz-ui-Breadcrumbs'></div>",
	
	postCreate: function() {
		this.refresh();
	},
	
	_createCrumb: function(obj){
		var span = document.createElement("span");
		var a = document.createElement("a");
		a.href = obj.href;
		a.appendChild(document.createTextNode(BidiComplex.createDisplayString(obj.label, "EMAIL")));
		if (obj.onClick) {
			this.connect(a,"onclick", function(event) {
				event.preventDefault();
				obj.onClick(event);
			});
		}
		span.appendChild(a);
		span.appendChild(document.createTextNode(" > "));
		return span;
	}
});

})();
