/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("jazz.core.CrossServer");

dojo.require("jazz.core.internal.CSSLoader");

/*
 * PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * future with minimal or no warning.
 * 
 * @since 0.7
 */

dojo.declare("jazz.core.CrossServer", null, {
	
	constructor: function(/*String*/serverUrl) {
		this.url = serverUrl;
		this.crossServerUrl = null;
		this.crossServerJAFUrl = null;
		
		var ore = new RegExp("^(([^:/?#]+):)?(//([^/?#]*))?([^?#]*)(\\?([^#]*))?(#(.*))?$");
		var ire = new RegExp("^((([^\\[:]+):)?([^@]+)@)?(\\[([^\\]]+)\\]|([^\\[:]*))(:([0-9]+))?$");
		
		var urlProps = this.url.match(ore);
		var protocol = urlProps[2] || (urlProps[1] ? "" : null);
		var authority = urlProps[4] || (urlProps[3] ? "" : null);
		
		if (authority !== null){
			var authorityProps = authority.match(ire);
			var host = authorityProps[6] || authorityProps[7]; // ipv6 || ipv4
			var port = authorityProps[9] || null;
			
			var originProtocol = window.location.protocol;
			if (originProtocol.charAt(originProtocol.length-1) == ':') {
				originProtocol = originProtocol.substring(0, originProtocol.length-1);
			}
			
			if (window.location.hostname == host &&
			    window.location.port == port &&
				originProtocol == protocol) {
				this.crossServerUrl = serverUrl;
			}
			else {
				var loc = window.location;
				this.crossServerUrl = loc.protocol + 
				                      "//" + 
									  loc.host + 
									  net.jazz.ajax._contextRoot + 
									  "/_proxy/" + 
									  encodeURIComponent(serverUrl);
			}
			this.crossServerJAFUrl = this.crossServerUrl + net.jazz.ajax._webuiPrefix + "net.jazz.ajax/_cross-server";
		}
		else {
			throw new Error("serverUrl argument has an invalid format ["+serverUrl+"]");
		}
	},
	
	connect: function() {
		var d = new dojo.Deferred();
		this._getCrossServerInfo(d);
		return d;
	},
	
	loadModule : function(/*String*/module) {
		if (this.crossServerUrl === null) { 
			throw new Error("serverUrl has not been set. This indicate that the requested server is not configured");
		}
		if (this.versionsMatch === false) {
			throw new Error("Version mismatch between servers");
		}
		if (!dojo.isString(module)) {
			throw new Error("module argument must be a String");	
		}
		
		var loadModuleURL = this.crossServerJAFUrl+"/loadmodule/"+module;
		var loadCSSURL = this.crossServerJAFUrl+"/loadcss/"+module+"?urlprefix="+encodeURIComponent(this.url);
		
		if (window.location.search.indexOf("debugDojo=true") > -1 ||
		    window.location.search.indexOf("debug=true") > -1) {
			loadModuleURL += "?debug=true";
			loadCSSURL += "&debug=true";
		}
		var d = new dojo.Deferred();
		var modules = dojo.dependency.moduleGraph.getValueList();
		var loadedModules = [];
		for (var i = 0; i < modules.length; i++) {
			if (modules[i].l === true) {
				loadedModules.push(modules[i].m);
			}
		}
		var bindArgs = {
			url: loadModuleURL,
			postData: dojo.toJson(loadedModules),
			preventCache: dojo.isIE,
			handleAs: "json",
			handle: function(response, ioArgs) {
				if (response instanceof Error) {
					d.errback(response);
				}
				else {
					try {
						var value = dojo["eval"](response);
						d.callback();
					}
					catch (e) {
						d.errback(e);
					}
				}
			}
		};
		var xhr = dojo.rawXhrPost(bindArgs);
		
		var loadedCSS = [];
		var cssModules = net.jazz.ajax.core.internal.style.CSSModuleMap.getAllCSSModules();
		for (i = 0; i < cssModules.length; i++) {
			if (cssModules[i].l === true) {
				loadedCSS.push(cssModules[i].u);
			}
		}
		bindArgs = {
			url: loadCSSURL,
			postData: dojo.toJson(loadedModules),
			preventCache: dojo.isIE,
			handleAs: "json",
			handle: function(cssModules, ioArgs) {
				if (cssModules instanceof Error) {
					d.errback(cssModules);
				}
				else {
					jazz.core.internal.CSSLoader._loadCSSModules(cssModules);
				}
			}
		};
		xhr = dojo.rawXhrPost(bindArgs);
		return d;
	},
/*	
	_getProviders: function(d) {
		var headers = {};
		headers["accept"] = "text/json";
		headers["content-type"] = "application/x-www-form-urlencoded; charset=utf-8";
		var self = this;
		var bindArgs = {
			url: dojo.baseUrl + "../../wspr",
			handleAs: "json",
			headers: headers,
			preventCache: dojo.isIE,
			handle: function(providers, ioArgs) {
				if (providers instanceof Error) {
					d.errback(providers);
				}
				else {
					dojo.forEach(providers, function(provider) {
						if (provider.remoteUri != self.url)	{return;}
						var loc = window.location;
						self.crossServerUrl = loc.protocol + 
						                     "//" + 
											 loc.host +
											 net.jazz.ajax._contextRoot +  
											 "/_proxy" + 
											 provider.localAlias;
						self.crossServerJAFUrl = self.crossServerUrl +					   
											 net.jazz.ajax._webuiPrefix + 
											 "net.jazz.ajax/_cross-server";
						self._getCrossServerInfo(d);
					});	
				}
			}
		};
		var xhr = dojo.xhrGet(bindArgs);
	},
*/	
	_getCrossServerInfo: function(/*dojo.Deferred*/d) {
		var headers = {};
		headers["accept"] = "text/json";
		headers["content-type"] = "application/x-www-form-urlencoded; charset=utf-8";
		this.versionsMatch = true;
		var self = this;
		var bindArgs = {
			url: self.crossServerJAFUrl,
			handleAs: "json",
			headers: headers,
			preventCache: dojo.isIE,
			handle: function(webBundles, ioArgs) {
				if (webBundles instanceof Error) {
					d.errback(webBundles);
				}
				else {
					dojo.forEach(webBundles, function(webBundle) {
						if (webBundle.namespace === "net.jazz.ajax" || webBundle.namespace === "org.dojotoolkit.dojo") {
							if (dojo.dependency.getWebBundleVersion(webBundle.namespace) != webBundle.version) {
								self.versionsMatch = false;
							}
						}
					});
					if (self.versionsMatch === true) {
						d.callback();
					}
					else {
						d.errback(new Error("Version mismatch between servers"));
					}
				}
			}
		};
		var xhr = dojo.xhrGet(bindArgs);
	}
});

})();